// READ THE COMMENTS FOR WHAT NEEDS TO BE DONE IN THIS TAB.

// First, here are some pre-program declarations and initializations.
Square[][] chessboard;

final color BLACK = #000000;
final color DARK_RED = #880000;
final color WHITE = #FFFFFF;

final int RANKS = 8;
final int FILES = 8;

PImage spriteSheet;
final int PIECE_TYPES = 6;
final int PIECE_COLORS = 2;
PImage[][] pieceSprites = new PImage[PIECE_TYPES][PIECE_COLORS];
final int IMAGE_WIDTH = 640;   // Sizes based on widths of piece images on spritesheet
final int IMAGE_HEIGHT = 213;
final int SPRITE_WIDTH = IMAGE_WIDTH / PIECE_TYPES;
final int SPRITE_HEIGHT = IMAGE_HEIGHT / PIECE_COLORS;

final int SQUARE_WIDTH = SPRITE_WIDTH;
final int SQUARE_HEIGHT = SPRITE_HEIGHT;

boolean showPieces = false;

/* 
 * A Processing program begins with the settings() and setup() methods,
 * then puts the draw() method on a sort of "infinite loop."
 */
void settings() {
  size((SQUARE_WIDTH + 1) * FILES + 2,
       (SQUARE_HEIGHT + 1) * RANKS + 2);
}

void setup() {
  initSquares();
  initPieces();
}

void draw() {
  background(BLACK);
  drawBoard(); 
  if (showPieces) drawPieces();
}

/*
 * initSquares() creates the Square objects and stores the references in
 * the chessboard 2D array.
 * NOTES: The lower left corner (position a1) must be a dark square.
 *        Squares should then alternate dark and white.
 *        Initially, pieces should be null.
 *        Add the pieces only after the board has been tested.
 */
void initSquares() {
  chessboard = new Square[RANKS][FILES];

  for (int r = 0; r < RANKS; r++) {
    for (int f = 0; f < FILES; f++) {
      // TODO: Create new Squares and store their references in the
      //       chessboard 2D array.
      // YOUR CODE HERE
      chessboard[r][f] = new Square(null, WHITE, r, f); // stub
    }
  }

  // Before a game, a chessboard should look like this:
  //
  //    black
  // 8 RNBQKBNR
  // 7 PPPPPPPP
  // 6 --------
  // 5 --------
  // 4 --------
  // 3 --------
  // 2 PPPPPPPP
  // 1 RNBQKBNR
  //   abcdefgh (files; ranks are vertical)
  //    white
  //
  // Note that indexing in arrays is 0..7, not 1..8 and definitely
  // not a..h, so you will have to keep track of that.
  // 
  // These lines of code place the Black King and a White Rook.
  // You can use these lines or comment them out
  chessboard[0][7].addPiece(new Piece("Rook", "White"));
  chessboard[7][4].addPiece(new Piece("King", "Black"));

  // TODO: Add pawns
  //       ranks 2 and 7 (rows 1 and 6)
  // HINT: Use nested loops!
  // YOUR CODE HERE
  

  // TODO: Add other pieces
  //       ranks 1 and 8 (rows 0 and 7)
  // HINT: Use a loop!
  // YOUR CODE HERE
  
}

/*
 * drawBoard() does the actual drawing of the squares based on
 * the information contained in the chessboard 2D array.
 */
void drawBoard() {
  for (int r = 0; r < RANKS; r++) {
    for (int f = 0; f < FILES; f++) {
      // TODO: Change the fill color to the square's color (objects in the Square
      //       class can tell you their color.)
      // YOUR CODE HERE
      fill(WHITE); // stub

      // TODO: Change the rect so it draws the current square
      // YOUR CODE HERE
      rect(0, 0, 0, 0); // stub
    }
  }
}

/*
 * initPieces() retrieves the sprites from the spriteSheet and stores
 * them in the pieceSprites 2D array[PIECE_TYPES][PIECE_COLORS]
 */
void initPieces() {
  // sprite source https://commons.wikimedia.org/wiki/File:Chess_Pieces_Sprite.svg
  // NOTE: In this sprite sheet, the pieces are ordered K, Q, B, N, R, P
  //       with the top row white and the bottom row black.
  spriteSheet = loadImage("640px-Chess_Pieces_Sprite.svg.png");

  for (int i = 0; i < PIECE_TYPES; i++) {
    for (int j = 0; j < PIECE_COLORS; j++) {
      pieceSprites[i][j] = spriteSheet.get(i * SPRITE_WIDTH, 
                                           j * SPRITE_HEIGHT,
                                           SPRITE_WIDTH,
                                           SPRITE_HEIGHT);
    }
  }
}

/*
 * drawPieces() does the actual drawing of the pieces based on
 * the information contained in the chessboard 2D array
 */
void drawPieces() {
  for (int i = 0; i < RANKS; i++) {
    for (int j = 0; j < FILES; j++) {
      if (chessboard[i][j].getPiece() != null) {
        // TODO: Get the piece color.
        // YOUR CODE HERE
        String pieceColor = null; // stub

        // TODO: Convert piece to index using typeToIndex() method.
        // YOUR CODE HERE
        int sheetRow = 5; // stub

        // TODO: Convert pieceColor to 0 if "White", else 1.
        // YOUR CODE HERE
        int sheetCol = 0; // stub

        // Draw sprites.
        image(pieceSprites[sheetRow][sheetCol], 
              1 + j * (SQUARE_WIDTH + 1), 
              1 + (FILES - i - 1) * (SQUARE_HEIGHT + 1));
      }
    }
  }
}

/*
 * typeToIndex() converts a Piece to an index in the spriteSheet
 * NOTE: See note above for order of piece types and colors.
 */
int typeToIndex(Piece piece) {
  switch(piece.getTypeChar()) {
  case 'K': 
    return 0; // King
  case 'Q': 
    return 1; // Queen
  case 'B': 
    return 2; // Bishop
  case 'N': 
    return 3; // kNight
  case 'R': 
    return 4; // Rook
  default: 
    return 5; // Pawn
  }
}
